major, minor, patch = (0, 1, 1)
__version__ = f"{major}.{minor}.{patch}"
__author__ = "Unai Saralegui"
__email__ = "usaralegui@gmail.com"
__credits__ = ["Unai Saralegui"]
__maintainer__ = "Unai Saralegui"
__status__ = "Development"
